import sys
sys.path.append("..")


from sklearn.datasets import load_digits
from sklearn.datasets import fetch_olivetti_faces
from sklearn.decomposition import FastICA
from sklearn import decomposition
from sklearn.cluster import MiniBatchKMeans
from sklearn import decomposition
import matplotlib.pyplot as plt
import skimage

import numpy as np
from numpy.random import RandomState

import logging
from time import time

from sklearn import linear_model
import torch

from scipy import signal
from scipy.signal import find_peaks
from sklearn.decomposition import PCA 

class FindComponents(object):
    """This class find the components in data. Those components could be independent or principal. It also returns the transformer.

    Args:
        object (object): Object of components and transformer
    """


    def __init__(self):
        """_summary_

        Args:
            train_data (np.array): _description_
            val_data (np.array): _description_
            image_shape (tuple, optional): _description_. Defaults to (64, 64).
            is_image (bool, optional): _description_. Defaults to True.
        """

        super(FindComponents, self).__init__()

        self.transformer = None
        self.ica_comps = None
        self.ica_comps_pix = None

    def clear_input(self):
        self.train_data = None
        self.val_data = None
        self.image_shape = None
        self.is_image = None
        

    def get_transformer_ica(self, train_data, val_data, image_shape=(64, 64), is_image=True, N_COMP_ICA=5, ICA_Method='logcosh'):
        """Takes number of data points and calculates the N Independent components based on sklearn.decomposition.FastICA. 
        Returns the transformer and corresponding ICA components in 2 verions (N ICA Components, and N ICA components per pixel.

        Args:
            train_data (np.array): _description_
            val_data (np.array): _description_
            image_shape (tuple, optional): _description_. Defaults to (64, 64).
            is_image (bool, optional): _description_. Defaults to True.
            N_COMP_ICA (int, optional): _description_. Defaults to 5.

        Returns:
            [transformer (object)]: transformer object for FastICA
            [ica_comps (list)]: FastICA N componenrs ordered for all pixels in single data
            [ica_comps_pix (list)]: FastICA N components ordered per pixel for all components in single data
        """

        self.train_data = train_data
        self.val_data = val_data
        self.image_shape = image_shape
        self.is_image = is_image

        self.train_data_points =  self.train_data.shape[0]
        if len(val_data.shape)==1:
            self.val_data_points = 1
        elif len(val_data.shape)!=1:
            self.val_data_points =  self.val_data.shape[0]

        assert(N_COMP_ICA<=self.train_data_points)

        transformer = FastICA(n_components=N_COMP_ICA, max_iter=5000, random_state=0, whiten=True, fun=ICA_Method)
        print("shape train data", self.train_data.shape)
        print("ICA_Method", ICA_Method)
        X_transformed = transformer.fit_transform(self.train_data)
        print("X_transformed.shape=", X_transformed.shape)

        n_img, l_img = transformer.mixing_.T.shape
        print("transformer.mixing_.T.shape", n_img, l_img)
        ica_comps = [transformer.mixing_[:, i] for i in range(N_COMP_ICA)]
        #print("ica_comps", ica_comps)


        # Get pixel wise N components by 
        ica_comps_pix = [transformer.mixing_[i, :] for i in range(l_img)]
        #print("ica_comps_pix", ica_comps_pix)

        self.transformer = transformer
        self.ica_comps = ica_comps
        self.ica_comps_pix = ica_comps_pix

        return self.transformer, self.ica_comps, self.ica_comps_pix, X_transformed


class FindNumberOfComponents(object):

    def __init__(self):
        self.comps = None
        self.PSDs = None
        self.PSDs_means = None
        self.user_n_components = None
        self.X_centered, self.y_centered = None, None

        self.n_good_comps = None
        self.good_comps = None
        
    
    def find_components(self):
        # Find components
        number_of_components = self.user_n_components
        fit_method = "BayesianRidge"

        FC = FindComponents()
        transformer, comps, comps_pix, X_transformed = FC.get_transformer_ica(train_data=self.X_centered, val_data=self.y_centered, N_COMP_ICA=self.user_n_components) # finds components
        self.comps = comps
    
    def find_PSDs(self):
        #self.find_components()
        self.PSDs = [signal.welch(ac)[1] for ac in self.comps]
        return self.PSDs

    def find_peaks_PSD(self):
        self.PSDs_means = np.mean(self.PSDs, axis=1)
        peaks, _ = find_peaks(self.PSDs_means, threshold=self.PSDs_means.min()*3)
        self.n_good_comps = len(peaks)
        self.good_comps =  peaks
        print("DONE !!!!!!!!!!!!!!!!!!")
        return self.n_good_comps, self.good_comps

    def n_good_components(self,
                         X_centered,
                         y_centered, 
                         number_of_components=20):
        
        self.user_n_components = number_of_components
        self.X_centered, self.y_centered = X_centered, y_centered
        
        self.find_components()
        self.find_PSDs()
        self.find_peaks_PSD()

        #plt.plot(self.PSDs_means)
        #plt.plot(self.good_comps, self.PSDs_means[self.good_comps], "*")
        #plt.plot(np.zeros_like(self.PSDs_means), "--", color="gray")
        #plt.show()
        #print(f"n_good_components = {self.n_good_comps}")
        #print(f"Good components = {self.good_comps}")
        

        return self.n_good_comps, self.good_comps
        

class FindPCAcomps(object):
    """_summary_

    Args:
        object (_type_): _description_
    """

    def __init__(self, data_train, data_test, n_pca_comps=50):
        self.pca_model = None
        self.data_train = data_train
        self.data_test = data_test
        self.n_pca_comps = n_pca_comps

    def clear_output(self):
        self.pca_model = None
        self.pca_train = None
        self.pca_test = None

    
    def get_pca(self):
        self.clear_output()
        self.pca_model = PCA(n_components=self.n_pca_comps, whiten=True)
        self.pca_train = self.pca_model.fit_transform(self.data_train)
        self.pca_test = self.pca_model.transform(self.data_test)

        return self.pca_model, self.pca_train, self.pca_test