from sklearn import linear_model
import torch
import scipy 
import h5py
import numpy as np
from autograd import numpy as anp   # Thinly-wrapped version of Numpy
from autograd import grad


        

class FitBFGSPCA_EPS(object):

    def __init__(self):

        self.X_train = None
        self.Y_train = None

        self.X_test = None
        self.Y_test = None

        self.Y_train_norm = None
        self.Y_test_norm = None

        self.spec_test = None

        self.Nx = None
        self.Ny = None
        self.A0 = None
        self.Aeps = None

        self.b0 = None
        self.u0 = None
        self.x0 = None

        self.sc_op = None
        self.A_inf = None
        self.b_inf = None
        self.u_inf = None

        self.model = {"A_inf":self.A_inf,
                      "b_inf":self.b_inf,
                      "u_inf":self.u_inf
                    
                      }
        
        self.pca_model_spec = None

        self.res = None
        self.res_unc = None
        self.res_pca_unc = None
        self.res_unc_specpca = None

        self.result = {"res":self.res,
                "res_unc":self.res_unc,
                "res_pca_unc": self.res_pca_unc,
                "res_unc_specpca": self.res_unc_specpca
                }

        self.input_data = None
        self.loss_train = None
        self.loss_test = None


    def loss_mae(self, x):
        # diag( (in @ x - out) @ (in @ x - out)^T )
        A = x[:self.Nx*self.Ny].reshape((self.Nx, self.Ny))
        b = x[self.Nx*self.Ny:(self.Nx*self.Ny+self.Ny)].reshape((1, self.Ny))
        log_unc = anp.clip(x[(self.Nx*self.Ny+self.Ny):].reshape((1, self.Ny)), -3, 8)
        #log_unc = x[(Nx*Ny+Ny):].reshape((1, Ny))
        iunc = anp.exp(-log_unc)
        #print("iunc2", iunc2)
        #print("log_unc", log_unc)

        return anp.mean((anp.fabs(self.X_train @ A + b - self.Y_train)*iunc + log_unc).sum(axis=1), axis=0 ) # Put RELU on (XX@x) and introduce new matrix W

    def loss_regul(self, x):
        # diag( (in @ x - out) @ (in @ x - out)^T )
        A = x[:self.Nx*self.Ny].reshape((self.Nx, self.Ny))
        b = x[self.Nx*self.Ny:(self.Nx*self.Ny+self.Ny)].reshape((1, self.Ny))
        #log_unc = anp.clip(x[(Nx*Ny+Ny):].reshape((1, Ny)), -3, 8)
        log_unc = x[(self.Nx*self.Ny+self.Ny):].reshape((1, self.Ny))
        iunc2 = anp.exp(-2*log_unc)
        unc2 = anp.exp(2*log_unc)
        #print("iunc2", iunc2)
        #print("log_unc", log_unc)
        width = 100.0
        w = 0.5/width**2
        #prior = w*((A**2).reshape((1, -1)).mean(axis=1))
    

        # New from Danilo->  p(unc|a, b) = ba/gamma(a) * unc(a-1) * np.exp(-b * unc)
        a = 3
        b = 6
        #  prior ->   - log p(unc|a, b) = - (a-1)*anp.log(unc) + b*unc
        prior = - (a-1) * log_unc + b * anp.exp(log_unc)


        return anp.mean( (0.5*((self.X_train @ A + b - self.Y_train_norm)**2)*iunc2 + log_unc).sum(axis=1) + prior.sum(axis=1), axis=0 ) # Put RELU on (XX@x) and introduce new matrix W

    def fill_init(self):


        self.Nx = self.X_train.shape[1]
        self.Ny = self.Y_train.shape[1]
        self.A0 = np.eye(self.Nx, self.Ny).reshape(self.Nx*self.Ny)
        self.Aeps = np.zeros(self.Nx)

        self.b0 = np.zeros(self.Ny)
        self.u0 = np.zeros(self.Ny)
        self.x0 = np.concatenate((self.A0,self.b0,self.u0,self.Aeps))

        


    def fit_eval(self, X_train, Y_train, X_test, Y_test):

        self.X_train = X_train
        self.Y_train = Y_train

        self.X_test = X_test
        self.Y_test = Y_test

        self.fill_init()


        def loss(x, X, Y):
            # diag( (in @ x - out) @ (in @ x - out)^T )
            A = x[:self.Nx*self.Ny].reshape((self.Nx, self.Ny))
            b = x[self.Nx*self.Ny:(self.Nx*self.Ny+self.Ny)].reshape((1, self.Ny))

            b_eps = x[(self.Nx*self.Ny+self.Ny):(self.Nx*self.Ny+self.Ny+self.Ny)].reshape((1, self.Ny))
            
            A_eps = x[(self.Nx*self.Ny+self.Ny+self.Ny):].reshape((self.Nx, 1))
            log_unc = X @ A_eps + b_eps

            #log_unc = anp.log(anp.exp(log_unc) + anp.exp(log_eps))
            iunc2 = anp.exp(-2*log_unc)
          
            #print("iunc2", iunc2)
            #print("log_unc", log_unc)

            return anp.mean( (0.5*((X@ A + b - Y)**2)*iunc2 + log_unc).sum(axis=1), axis=0 ) # Put RELU on (XX@x) and introduce new matrix W

        self.loss_train = []
        self.loss_test = []

        def loss_hist(x):
            l_train = loss(x, X_train, self.Y_train)
            l_test = loss(x, X_test, self.Y_test)

            self.loss_train.append(l_train)
            self.loss_test.append(l_test)
            return l_train

        def loss_hist_2(x):
            l_train = loss(x, self.X_train, self.Y_train)
            return l_train

        grad_loss = grad(loss_hist_2)
        #self.grad_loss_regul = grad(loss_regul)
        self.sc_op = scipy.optimize.fmin_l_bfgs_b(loss_hist, self.x0, grad_loss, disp=True, factr=1e7, maxiter=100, iprint = 0)

        # Inference
        #print(sc_res.shape)
        self.A_inf = self.sc_op[0][:self.Nx*self.Ny].reshape(self.Nx, self.Ny)
        self.b_inf = self.sc_op[0][self.Nx*self.Ny:(self.Nx*self.Ny+self.Ny)].reshape(1, self.Ny)
        self.u_inf = self.sc_op[0][(self.Nx*self.Ny+self.Ny):(self.Nx*self.Ny+self.Ny+self.Ny)].reshape(1, self.Ny) # removed np.exp
        self.A_eps = self.sc_op[0][(self.Nx*self.Ny+self.Ny+self.Ny):].reshape(self.Nx, 1)

        self.model = {"A_inf":self.A_inf,
                      "b_inf":self.b_inf,
                      "u_inf":self.u_inf,
                      "A_eps":self.A_eps,
                      "loss_train": self.loss_train,
                      "loss_test": self.loss_test,
                      }

        return self.model

    def save_preproc(self,
                        pes_train, pes_test,
                        spec_train, spec_test,
                        X_train, X_test,
                        Y_train, Y_test,
                        spec_raw_pe,
                        data_info,
                        xgm_pulseen_train,
                        xgm_pulseen_test
                        ):

        exp_dir = data_info["exp_dir"]

        hf = h5py.File(f"experiments/{exp_dir}/output/data.h5", 'w')
        hf.create_dataset('pes_train', data=pes_train)
        hf.create_dataset('pes_test', data=pes_test)
        hf.create_dataset('spec_train', data=spec_train)
        hf.create_dataset('spec_test', data=spec_test)
        hf.create_dataset('X_train', data=X_train)
        hf.create_dataset('X_test', data=X_test)
        hf.create_dataset('Y_train', data=Y_train)
        hf.create_dataset('Y_test', data=Y_test)
        hf.create_dataset('spec_raw_pe', data=spec_raw_pe[0,:])
        hf.create_dataset('xgm_pulseen_train', data=xgm_pulseen_train)
        hf.create_dataset('xgm_pulseen_test', data=xgm_pulseen_test)
        
        for key, val in data_info.items():
            hf.attrs[key] = val

    def save_latest_chp(self, exp_dir):
            filemodel = f"experiments/{exp_dir}/checkpoints/chp.h5"
            hf = h5py.File(filemodel, 'w')

            for k, v in self.model.items():
                print("saving:", k)
                hf.create_dataset(k, data=v)
            hf.close()
            print(f"model saved in -> {filemodel}")

    def load_model(self, exp_dir):

        # Load Checkpoint Data
        print("loading model", exp_dir)
        filename = f"experiments/{exp_dir}/checkpoints/chp.h5"
        file = h5py.File(filename, 'r')
        A_inf = file["A_inf"][()]  # returns as a numpy array
        b_inf = file["b_inf"][()]  # returns as a numpy array
        u_inf = file["u_inf"][()]  # returns as a numpy array
        A_eps = file["A_eps"][()]  # returns as a numpy array


        import joblib
        pes_pca_model=joblib.load(f"experiments/{exp_dir}/checkpoints/pes_pca_model.joblib")
        spec_pca_model=joblib.load(f"experiments/{exp_dir}/checkpoints/spec_pca_model.joblib")

        self.model = {"A_inf":A_inf,
                "b_inf":b_inf,
                "u_inf":u_inf,
                "A_eps":A_eps,
                "pes_pca_model":pes_pca_model,
                "spec_pca_model":spec_pca_model

                }
        return self.model, pes_pca_model, spec_pca_model

    def predict(self, input_data, input_target, spec_target):

        if self.pca_model_spec is not None:
            self.model["spec_pca_model"] = self.pca_model_spec # TODO! check for training

        if input_target is None:
            self.model["Y_test"] = self.Y_test
        else:
            self.model["Y_test"] = input_target

        if spec_target is None:
            self.model["spec_target"] = self.spec_test
        else:
            self.model["spec_target"] = spec_target





        self.result["res_pca"] = (input_data @ self.model["A_inf"] + self.model["b_inf"])   # inference reconstruction in PCA space
        self.result["res_pca_unc"] = self.model["u_inf"][0,:]             # unc in pca space
        self.result["res_pca_eps"] = np.exp(input_data @ self.model["A_eps"] + self.result["res_pca_unc"])   # eps in pca space

        self.result["res"] = self.model["spec_pca_model"].inverse_transform(self.result["res_pca"]) # transform PCA space to real space
        
        self.result["res_unc"] = self.model["spec_pca_model"].inverse_transform(self.result["res_pca"] + self.model["u_inf"]) - self.model["spec_pca_model"].inverse_transform(self.result["res_pca"] ) 
        self.result["res_unc"] = np.fabs(self.result["res_unc"])

        self.result["res_eps"] = self.model["spec_pca_model"].inverse_transform(self.result["res_pca"] + self.result["res_pca_eps"]) - self.model["spec_pca_model"].inverse_transform(self.result["res_pca"] ) 
        self.result["res_eps"] = np.fabs(self.result["res_eps"])


        self.Yhat_pca = self.model["spec_pca_model"].inverse_transform(self.model["Y_test"])
        self.result["res_unc_specpca"] =  np.sqrt(((self.Yhat_pca - self.model["spec_target"])**2).mean(axis=0))

        self.result["res_unc_total"] =  np.sqrt(self.result["res_eps"]**2 + self.result["res_unc_specpca"]**2)
        return self.result


    def save_prediction(self, exp_dir):
    
        fileprediction = f"experiments/{exp_dir}/output/pred_data.h5"
        hf = h5py.File(fileprediction, 'w')
        hf.create_dataset('res_pca', data=self.result["res_pca"])
        hf.create_dataset('res_pca_unc', data=self.result["res_pca_unc"])
        hf.create_dataset('res', data=self.result["res"])
        hf.create_dataset('res_unc', data=self.result["res_unc"])
        hf.create_dataset('res_unc_specpca', data=self.result["res_unc_specpca"])
        hf.create_dataset('res_unc_total', data=self.result["res_unc_total"])
        hf.create_dataset('res_pca_eps', data=self.result["res_pca_eps"])
        hf.create_dataset('res_eps', data=self.result["res_eps"])

        hf.close()

        print(f"Prediction result saved in -> {fileprediction}")
        

